#!/usr/bin/perl

# Copyright (C) 2005 John J. Chew, III <jjchew@math.utoronto.ca>
# All Rights Reserved

package TSH::Command::KOTH;

use strict;
use warnings;

use TSH::PairingCommand;
use TSH::Player;
use TSH::Utility qw(Debug);

our (@ISA) = qw(TSH::PairingCommand);

=pod

=head1 NAME

TSH::Command::KOTH - implement the C<tsh> KOTH command

=head1 SYNOPSIS

  my $command = new TSH::Command::KOTH;
  my $argsp = $command->ArgumentTypes();
  my $helptext = $command->Help();
  my (@names) = $command->Names();
  $command->Run($tournament, @parsed_arguments);
  
=head1 ABSTRACT

TSH::Command::KOTH is a subclass of TSH::Command.

=cut

=head1 DESCRIPTION

=over 4

=cut

sub initialise ($$$$);
sub new ($);
sub Run ($$@);

=item $parserp->initialise()

Used internally to (re)initialise the object.

=cut

sub initialise ($$$$) {
  my $this = shift;
  my $path = shift;
  my $namesp = shift;
  my $argtypesp = shift;

  $this->{'help'} = <<'EOF';
Use the KOTH command to manually add a round of king-of-the-hill pairings
to a division, specifying the maximum number of allowable repeats,
and the round on whose standings the pairings are to be based.
Players are paired with opponents ranked as close as possible, with
ties broken by avoiding consecutive repeats, pairing starters with
repliers, and minimizing repeats.
EOF
  $this->{'names'} = [qw(koth)];
  $this->{'argtypes'} = [qw(RepeatsSince BasedOnRound Division)];
# print "names=@$namesp argtypes=@$argtypesp\n";

  return $this;
  }

sub new ($) { return TSH::Utility::new(@_); }

=item $command->Run($tournament, @parsed_args)

Should run the command in the context of the given
tournament with the specified parsed arguments.

=cut

sub Run ($$@) { 
  my $this = shift;
  my $tournament = shift;
  my ($repeats, $since1, $sr, $dp) = @_;
  my $sr0 = $sr-1;
  my $since0 = $since1 - 1;

  {
    my $r0 = $dp->FirstUnpairedRound0();
    $dp->ComputeRanks($dp->FirstUnpairedRound0()-1) 
      if $r0 > -1;
  }
  my $setupp = $this->SetupForPairings(
    'division' => $dp, 'source0' => $sr0, 'repeats' => $repeats, 'repsince0' => $since0) or return 0;
  
  if (TSH::Player::PairGRT($setupp->{'players'},
    # opponent preference ranking
    # $psp is arg 0
    # $pindex is arg 1
    # $oppindex is arg 2
    sub {
      my $p = $_[0][$_[1]];
      my $pid = $p->ID();
      my $o = $_[0][$_[2]];
      my $oid = $o->ID();
      my $lastoid = ($p->OpponentID(-1)||-1);
      my $thisrep = $p->Repeats($oid);
      $thisrep -= $p->CountRoundRepeats($o, $since0) if defined $since0;
      my $sameopp = ($oid == $lastoid);
      my $distance = abs($_[1]-$_[2]);
      my $pairsvr = $config::track_firsts ? 2-abs(($p->{'p1'}-$p->{'p2'} <=> 0)  -($o->{'p1'}-$o->{'p2'} <=> 0)) : 0;

      Debug 'GRT', 'pref %d-%d rep=%d prev=%d svr=%d dist=%d', $pid, $oid, $thisrep, $sameopp, $pairsvr, $distance;
      pack('NCCNN',
	$distance, # prefer correctly positioned opponents
	$sameopp, # avoid previous opponent
	$pairsvr, # pair those due to start vs those due to reply
 	$thisrep, # minimize repeats
	$_[2], # to recover player ID
	)
      },
    # allowed opponent filter
    $setupp->{'filter'},
    # optional arguments to subs
    [],
    # target round
    $setupp->{'target0'},
    # do not optimize order in which player opponent search is
    # performed, as this may pessimize pairings, especially when
    # byes are involved
    { 'optimize' => scalar(@{$setupp->{'players'}}) },
    )) {
    $this->TidyAfterPairing($dp);
    }
  else {
    $tournament->TellUser('epfail');
    }
  }

=back

=cut

=head1 BUGS

Makes inappropriate use of private functions in TSH::Player.

Should just call newer FactorPair command.

=cut

1;
